@@ -5,32 +5,38 @@ import numpy as np
 import matplotlib.pyplot as plt
 import matplotlib as mpl
 from mpl_toolkits.mplot3d import Axes3D
-import sys
 from .TomoDisplayHelpers import *
+from .TomoFunctions import removeGlobalPhase
+from matplotlib.colors import LinearSegmentedColormap
+import warnings
 
 """
 Copyright 2020 University of Illinois Board of Trustees.
 Licensed under the terms of an MIT license
 """
 
-__author__ = 'Quoleon/Turro'
+
 """CHECK OUT THE REFERENCE PAGE ON OUR WEBSITE :
-http://research.physics.illinois.edu/QI/Photonics/Quantum-Tomography_lib_Ref/"""
+https://quantumtomo.web.illinois.edu/Doc/"""
 
 
 """
-    makeRhoImages(p, plt_given, customColor)
-    Desc: Creates matlab plots of the density matrix.
+makeRhoImages(p, plt_given, customColor)
+Desc: Creates matlab plots of the density matrix.
 
-    Parameters
-    ----------
-    p : ndarray with shape = (n, 2^numQubits, 2^numQubits)
-        The density matrix you want to create plots of.
-    plt_given : matplotlib.pyplot
-        Input pyplot for which the figures will be saved on to.
-    customColor : boolean
-        Specify if you want our custom colorMap. Default is true
-    """
+Parameters
+----------
+p : ndarray with shape = (n, 2^numQubits, 2^numQubits)
+    The density matrix you want to create plots of.
+plt_given : matplotlib.pyplot
+    Input pyplot for which the figures will be saved on to.
+customColor : boolean
+    Specify if you want our custom colorMap. Default is true
+
+See Also
+ ------ 
+saveRhoImages
+"""
 def makeRhoImages(p, plt_given, customColor = True):
     # Set up
     numQubits = int(np.log2(p.shape[0]))
@@ -115,19 +121,21 @@ def makeRhoImages(p, plt_given, customColor = True):
                                     orientation = 'horizontal')
 
 """
-    saveRhoImages(p, pathToDirectory, customColor)
-    Desc: Creates and saves matlab plots of the density matrix.
+saveRhoImages(p, pathToDirectory, customColor)
+Desc: Creates and saves matlab plots of the density matrix.
 
-    Parameters
-    ----------
-    p : ndarray with shape = (n, 2^numQubits, 2^numQubits)
-        The density matrix you want to create plots of.
-    pathToDirectory : string
-        Path to where you want your images to be saved.
-    customColor : boolean
-        Specify if you want our custom colorMap. Default is true.
-    """
-def saveRhoImages(p, pathToDirectory, customColor = True):
+Parameters
+----------
+p : ndarray with shape = (n, 2^numQubits, 2^numQubits)
+    The density matrix you want to create plots of.
+pathToDirectory : string
+    Path to where you want your images to be saved.
+
+See Also
+ ------ 
+makeRhoImages
+"""
+def saveRhoImages(p, pathToDirectory):
     # Set up
     numQubits = int(np.log2(p.shape[0]))
     xpos = np.zeros_like(p.flatten(), dtype = float)
@@ -142,19 +150,16 @@ def saveRhoImages(p, pathToDirectory, customColor = True):
     dy = .9 * np.ones_like(ypos)
     # custom color map
     n_bin = 100
-    if (customColor):
-        from matplotlib.colors import LinearSegmentedColormap
-        cmap_name = 'my_list'
-        colors = [(1 / 255.0, 221 / 255.0, 137 / 255.0),
-                  (32 / 255.0, 151 / 255.0, 138 / 255.0),
-                  (53 / 255.0, 106 / 255.0, 138 / 255.0),
-                  (86 / 255.0, 33 / 255.0, 139 / 255.0),
-                  (131 / 255.0, 75 / 255.0, 114 / 255.0),
-                  (173 / 255.0, 114 / 255.0, 90 / 255.0),
-                  (253 / 255.0, 187 / 255.0, 45 / 255.0)]
-        colorMap = LinearSegmentedColormap.from_list(cmap_name, colors, N = n_bin)
-    else:
-        colorMap = plt.cm.jet
+    cmap_name = 'my_list'
+    colors = [(1 / 255.0, 221 / 255.0, 137 / 255.0),
+              (32 / 255.0, 151 / 255.0, 138 / 255.0),
+              (53 / 255.0, 106 / 255.0, 138 / 255.0),
+              (86 / 255.0, 33 / 255.0, 139 / 255.0),
+              (131 / 255.0, 75 / 255.0, 114 / 255.0),
+              (173 / 255.0, 114 / 255.0, 90 / 255.0),
+              (253 / 255.0, 187 / 255.0, 45 / 255.0)]
+    colorMap = LinearSegmentedColormap.from_list(cmap_name, colors, N = n_bin)
+
     norm = mpl.colors.Normalize(vmin = -1, vmax = 1)
 
     tickBase = ["H", "V"]
@@ -206,21 +211,21 @@ def saveRhoImages(p, pathToDirectory, customColor = True):
     cb1 = mpl.colorbar.ColorbarBase(ax1, cmap = colorMap,
                                     norm = norm,
                                     orientation = 'horizontal')
+
     plt.savefig(pathToDirectory + "/rhobarImag.png", bbox_inches = 'tight', pad_inches = 0)
 
-"""
-    printLastOutput(tomo, bounds)
-    Desc: Prints the properties of the last tomography to the console. Properties are defined in tomography conf settings.
-          Using bounds will not change the conf settings. The calculated properties are determined by self.err_functions.
 
-    Parameters
-    ----------
-    tomo : Tomography() Object
-        This is the main tomography object. It will get the data from it's last tomography and print it.
-    bounds : boolean
-        Set this to true if you want error bounds on your estimated property values. Default is False.
-        These are determined with monte carlo simulation and the states are saved under self.mont_carl_states
-    """
+"""
+printLastOutput(tomo,bounds)
+Desc: Prints the properties of the last tomography to the console. Properties are defined in tomography conf settings. The calculated properties are determined by self.err_functions.
+
+Parameters
+----------
+tomo : Tomography
+    The tomography object you want to see the output of.
+bounds : int (optional)
+    The number of monte carlo runs you want to perform to get a better estimate of each property. Default will use whatever is set in the conf settings.
+"""
 def printLastOutput(tomo, bounds = -1):
     p = np.array(tomo.last_rho.copy(), dtype = "O")
     print("State: ")
@@ -237,29 +242,32 @@ def printLastOutput(tomo, bounds = -1):
         print("")
 
     # print(p)
-    properties = tomo.getProperties(tomo.last_rho, bounds)
+    properties = tomo.getProperties(bounds)
     for prop in properties:
-        if(len(prop) >3):
+        if(len(prop) >=3) and prop[2] != 'NA':
             print(prop[0] + " : " + floatToString(prop[1]) + " +/- " + floatToString(prop[2]))
         else:
             print(prop[0] + " : " + floatToString(prop[1]))
 
 """
-    matrixToHTML(M)
-    Desc: Creates an HTML table based on the given matrix.
+matrixToHTML(M)
+Desc: Creates an HTML table based on the given matrix.
 
-    Parameters
-    ----------
-    M : 2d numpy array with shape = (2^numQubits, 2^numQubits)
-        Matrix you would like to display on your html page.
-    printEigenVals : boolean
-        Specify if you want eigen values to be calculated and displayed at the bottom of the table.
-    Returns
-    -------
-    res : string
-        HTML code of the created table.
-    """
+Parameters
+----------
+M : 2d numpy array with shape = (2^numQubits, 2^numQubits)
+    Matrix you would like to display on your html page.
+printEigenVals : boolean
+    Specify if you want eigen values to be calculated and displayed at the bottom of the table.
+Returns
+-------
+res : string
+    HTML code of the created table.
 
+See Also
+ ------ 
+propertiesToHTML
+"""
 def matrixToHTML(M, printEigenVals = False):
     s = np.shape(M)
     res = '<table class="KwiatDataMatrix" style = \"border: 1px solid black;border-collapse: collapse;font-size: 15px; table-layout:fixed;width:100%;margin-top: 25px;\">'
@@ -274,7 +282,7 @@ def matrixToHTML(M, printEigenVals = False):
     if(printEigenVals):
         d, v = np.linalg.eig(M)
         sum = 0
-        eigenVals = "<h5>Eigen Values: "
+        eigenVals = "<h5>Eigen Values : "
         for x in range(0, len(d)):
             eigenVals = eigenVals+str(round(d[x].real, 5))
             if(abs(d[x].imag)>.00001):
@@ -284,43 +292,64 @@ def matrixToHTML(M, printEigenVals = False):
             eigenVals = eigenVals+" , "
             sum+= d[x]
         eigenVals = str(eigenVals)[0:len(str(eigenVals))-2]
-        eigenVals = eigenVals+" = "+floatToString(sum, True)+"</h5>"
+        eigenVals = eigenVals +"</h5>"
         res = res+eigenVals
     return res
 
 
 """
-    propertiesToHTML(vals)
-    Desc: Creates an HTML table based on the given property values.
+propertiesToHTML(vals)
+Desc: Creates an HTML table based on the given property values.
 
-    Parameters
-    ----------
-    vals : ndarray with shape = (length of self.err_functions, 2)
-        The first col is the name of the property.
-        The second col is the value of the property.
-        The third col is the error bound on the property.
+Parameters
+----------
+vals : ndarray with shape = (length of self.err_functions, 2)
+    The first col is the name of the property.
+    The second col is the value of the property.
+    The third col is the error bound on the property.
 
-    Returns
-    -------
-    res : string
-        HTML code of the created table.
-    """
+Returns
+-------
+res : string
+    HTML code of the created table.
 
+See Also
+ ------ 
+matrixToHTML
+"""
 def propertiesToHTML(vals):
-    f = '<h3 >Properties of Rho</h3><table style = \"width:60%;margin-top:10px;font-size: 15px;padding-bottom:5px;float:none;\"><tr><td style = "font-size: 20;font-weight: 1000;color: rebeccapurple;">Property</td><td  style = "font-size: 20;font-weight: 1000;color: rebeccapurple;padding-bottom:5px;">Value</td>'
-    if(vals.shape[1] < 3):
+    f = '<h3 >Properties of Rho</h3><table style = \"width:60%;margin-top:10px;font-size: 15px;padding-bottom:5px;float:none;\"><tr><td style = "font-size: 20;font-weight: 1000;color: rebeccapurple;">Property</td>'
+    hasSTD = len(vals) > 2
+    if hasSTD and any(vals[:,2] != "NA"):
+        f += '<td  style = "font-size: 20;font-weight: 1000;color: rebeccapurple;padding-bottom:5px;">Average Value</td>'
+        f += '<td style = "font-size: 20;font-weight: 1000;color: rebeccapurple;padding-bottom:5px;">STD Error</td></tr>'
         f += '<td style = "font-size: 20;font-weight: 1000;color: rebeccapurple;padding-bottom:5px;"></td></tr>'
     else:
-        f += '<td style = "font-size: 20;font-weight: 1000;color: rebeccapurple;padding-bottom:5px;"> STD Deviation </td></tr>'
+        f += '<td  style = "font-size: 20;font-weight: 1000;color: rebeccapurple;padding-bottom:5px;">Value</td>'
+        f += '<td style = "font-size: 20;font-weight: 1000;color: rebeccapurple;padding-bottom:5px;"></td></tr>'
     for v in vals:
         if(v[1] != "NA"):
             f += '<tr>'
             f += '<td><div onmouseover = "Tip( ' + v[0].replace(" ", "") + 'Tip)" onmouseout = "hideTip()">'+v[0] + '</td>'
             f += '<td name = "' + v[0].replace(" ", "") + '_value">' + floatToString(v[1], True) + '</td>'
-            if (len(v) > 2 and v[2] != ""):
+            if hasSTD and v[2] != "NA":
                 f += '<td> +/- ' + floatToString(v[2], True) + '</td>'
             else:
                 f += "<td></td>"
             f += '</tr>'
     f += "</table>"
     return f
+
+
+"""
+stateToString(vals)
+Desc: Creates a string of the 1d pure state.
+
+Parameters
+----------    
+pure_state : 1darray with length = 2
+    The state in ket form.
+"""
+def stateToString(state):
+    state = removeGlobalPhase(state)
+    return floatToString(state[0])+"|H> + "+floatToString(state[1])+"|V>"
